var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as dotenv from 'dotenv';
import * as rslConstants from './smrtfill.constants.js';
import { exceptionalError } from './smrtfill.js';
import { logger } from './smrtfill.js';
import util from 'util';
dotenv.config();
let rslBaseUrl = "";
let rslClientId = "";
let rslApiId = "";
let rslPW = "";
const isProduction = (process.env.NODE_ENV === 'PRODUCTION');
if (isProduction) {
    rslBaseUrl = process.env.PROD_RSL_BASE_URL;
    rslClientId = process.env.PROD_RSL_PROD_CLIENT_ID;
    rslApiId = process.env.PROD_RSL_API_ID;
    rslPW = process.env.PROD_RSL_PW;
}
else {
    rslBaseUrl = process.env.DEV_RSL_BASE_URL;
    rslClientId = process.env.DEV_RSL_PROD_CLIENT_ID;
    rslApiId = process.env.DEV_RSL_API_ID;
    rslPW = process.env.DEV_RSL_PW;
}
export let token = null;
export let expiry = null;
export let connected = false;
export function performApiCall(endPoint, method, body = null) {
    return __awaiter(this, void 0, void 0, function* () {
        let fetchOpts = {
            headers: {}
        };
        yield validateAPICall(endPoint, method);
        fetchOpts.method = method.toUpperCase();
        if (body && fetchOpts.method === rslConstants.methodPost) {
            fetchOpts.body = JSON.stringify(body);
            fetchOpts.headers['Content-Type'] = 'application/json';
        }
        if (endPoint !== rslConstants.endpointAuth) {
            fetchOpts.headers.Authorization = `Bearer ${token}`;
            fetchOpts.headers.ClientID = rslClientId;
        }
        const reqUrl = `${rslBaseUrl}${endPoint}`;
        return yield processResponse(reqUrl, fetchOpts);
    });
}
// Invalidate token - mainly for testing purposes
export function disconnect() {
    token = null;
    expiry = null;
    connected = false;
}
function validateAPICall(endPoint, method) {
    return __awaiter(this, void 0, void 0, function* () {
        if (endPoint !== rslConstants.endpointAuth) {
            yield checkToken();
        }
        if (![rslConstants.methodPost, rslConstants.methodGet].includes(method.toUpperCase())) {
            throw new exceptionalError(`Unknown method ${method}`);
        }
        if (endPoint.match(/^\//)) {
            throw new exceptionalError(`No leading slashes in the endpoint URL!`);
        }
    });
}
function checkToken() {
    return __awaiter(this, void 0, void 0, function* () {
        const nowDate = Number((new Date()));
        if (!token || !expiry || (expiry - nowDate <= rslConstants.refreshTokenThreshold)) {
            token = null;
            expiry = null;
            connected = false;
            yield authenticate();
        }
    });
}
export function authenticate() {
    return __awaiter(this, void 0, void 0, function* () {
        const authBody = {
            clientId: rslClientId,
            apiUserIdentifier: rslApiId,
            apiUserSecret: rslPW
        };
        const authReq = yield performApiCall(rslConstants.endpointAuth, rslConstants.methodPost, authBody);
        if (!authReq.data || !authReq.data.token) {
            logger.info(util.inspect(authReq));
            throw new exceptionalError("Didnt get a token somehow?");
        }
        token = authReq.data.token;
        connected = true;
        expiry = Number((new Date())) + rslConstants.tokenLifespan;
    });
}
function processResponse(reqUrl, fetchOpts) {
    return __awaiter(this, void 0, void 0, function* () {
        const apiResp = yield fetch(reqUrl, fetchOpts);
        logger.info(`RSL API: ${reqUrl} | ${fetchOpts.method} | Returned: ${apiResp.status}`);
        switch (apiResp.status) {
            case 200:
                return yield apiResp.json();
                break;
            case 400:
                yield handleError(apiResp);
                break;
            case 404:
                yield handleError(apiResp);
                break;
            case 202:
                return true;
            default:
                throw new exceptionalError(`Can't handle ${apiResp.status} right now!`);
        }
    });
}
function handleError(apiResp) {
    return __awaiter(this, void 0, void 0, function* () {
        let errs = [];
        if (apiResp.body) {
            const respBody = yield apiResp.json();
            if (respBody.status && respBody.status.statusCodes) {
                for (const errEntry of respBody.status.statusCodes) {
                    let thisErr = [];
                    if (errEntry.statusCode) {
                        thisErr.push(`Error ${errEntry.statusCode}`);
                    }
                    if (errEntry.statusMessage) {
                        thisErr.push(`${errEntry.statusMessage}`);
                    }
                    if (thisErr.length > 0) {
                        errs.push(thisErr.join(" "));
                    }
                }
            }
        }
        else {
            let err = `Error ${apiResp.status}`;
            if (apiResp.statusText) {
                err = `${err} - ${apiResp.statusText}`;
            }
            errs.push(err);
        }
        let errString = "";
        if (errs.length === 0) {
            errString = `Request returned ${apiResp.status}`;
            if (apiResp.statusText) {
                errString = `${errString} ${apiResp.statusText}`;
            }
        }
        else {
            errString = `Request failure: \n  -> ${errs.join(`\n  -> `)}`;
        }
        logger.info(util.inspect(apiResp.body));
        throw new exceptionalError(errString);
    });
}
export function overrideExpTime(newTime) {
    if (isProduction) {
        throw new exceptionalError("This aint for you production!");
    }
    expiry = newTime;
}
export function rawRequest(endpoint, method) {
    return __awaiter(this, void 0, void 0, function* () {
        if (isProduction) {
            throw new exceptionalError("This aint for you production!");
        }
        return yield performApiCall(endpoint, method, {});
    });
}
