import * as dotenv from 'dotenv'
import * as rslConstants from './smrtfill.constants.js';
import {exceptionalError} from './smrtfill.js';
import {logger} from './smrtfill.js'
import util from 'util'

dotenv.config();

let rslBaseUrl: string = "";
let rslClientId: string = "";
let rslApiId: string = "";
let rslPW: string = "";
const isProduction = (process.env.NODE_ENV === 'PRODUCTION')

if (isProduction){
    rslBaseUrl = process.env.PROD_RSL_BASE_URL as string;
    rslClientId = process.env.PROD_RSL_PROD_CLIENT_ID as string;
    rslApiId = process.env.PROD_RSL_API_ID as string;
    rslPW = process.env.PROD_RSL_PW as string;
   
} else {
    rslBaseUrl = process.env.DEV_RSL_BASE_URL as string;
    rslClientId = process.env.DEV_RSL_PROD_CLIENT_ID as string;
    rslApiId = process.env.DEV_RSL_API_ID as string;
    rslPW = process.env.DEV_RSL_PW as string;
}

export let token: null|string = null;
export let expiry: null|number = null;
export let connected: boolean = false;


export async function performApiCall(endPoint: string, method: string, body: rslConstants.authBody|any|null = null){
    let fetchOpts: rslConstants.fetchOpts = {
        headers: {}
    }; 
    await validateAPICall(endPoint, method);
    fetchOpts.method = method.toUpperCase();
    
    if(body && fetchOpts.method === rslConstants.methodPost){
        fetchOpts.body = JSON.stringify(body)
        fetchOpts.headers!['Content-Type'] = 'application/json'
    }

    if(endPoint !== rslConstants.endpointAuth){
        fetchOpts.headers!.Authorization = `Bearer ${token}`;
        fetchOpts.headers!.ClientID = rslClientId;
    }
    const reqUrl = `${rslBaseUrl}${endPoint}`
    return await processResponse(reqUrl, fetchOpts); 
}

// Invalidate token - mainly for testing purposes
export function disconnect(){
    token = null;
    expiry = null;
    connected = false;
}

async function validateAPICall(endPoint: string, method: string){
    if(endPoint !== rslConstants.endpointAuth){
        await checkToken();
    }
    if (![rslConstants.methodPost, rslConstants.methodGet].includes(method.toUpperCase())){
        throw new exceptionalError(`Unknown method ${method}`)
    }
    if (endPoint.match(/^\//)){
        throw new exceptionalError(`No leading slashes in the endpoint URL!`)
    }
}


async function checkToken(){
    const nowDate = Number((new Date()));
    if(!token || !expiry || (expiry - nowDate <= rslConstants.refreshTokenThreshold )){
        token = null;
        expiry = null;
        connected = false;
        await authenticate();
    }
} 


export async function authenticate(){
    const authBody: rslConstants.authBody = {
        clientId: rslClientId,
        apiUserIdentifier: rslApiId,
        apiUserSecret: rslPW
    }
    const authReq = await performApiCall(rslConstants.endpointAuth, rslConstants.methodPost, authBody) as rslConstants.tokenResp
    if(!authReq.data || !authReq.data.token){
        logger.info(util.inspect(authReq));
        throw new exceptionalError("Didnt get a token somehow?")
    }
    token = authReq.data.token;
    connected = true;
    expiry = Number((new Date())) + rslConstants.tokenLifespan;
}


async function processResponse(reqUrl: string, fetchOpts: rslConstants.fetchOpts){
    const apiResp = await fetch(reqUrl, fetchOpts)
    logger.info(`RSL API: ${reqUrl} | ${fetchOpts.method} | Returned: ${apiResp.status}`)
    switch(apiResp.status){
        case 200:
            return await apiResp.json();
            break;
        case 400:
            await handleError(apiResp);
            break;
        case 404: 
            await handleError(apiResp);
            break;
        case 202:
            return true
        default:
            throw new exceptionalError(`Can't handle ${apiResp.status} right now!`)
    }
}


async function handleError(apiResp: Response){
    let errs: string[] = [];
    if (apiResp.body){
        const respBody = await apiResp.json();
        if(respBody.status && respBody.status.statusCodes){
            for(const errEntry of respBody.status.statusCodes){
                let thisErr: string[] = [];
                if(errEntry.statusCode){
                    thisErr.push(`Error ${errEntry.statusCode}`)
                }
                if(errEntry.statusMessage){
                    thisErr.push(`${errEntry.statusMessage}`)
                }
                if(thisErr.length > 0){
                    errs.push(thisErr.join(" "));
                }
            }
        }
    } else {
        let err = `Error ${apiResp.status}`
        if(apiResp.statusText){
            err = `${err} - ${apiResp.statusText}`
        }
        errs.push(err); 
    }
    let errString = "";
    if(errs.length === 0){
        errString = `Request returned ${apiResp.status}`
        if(apiResp.statusText){
            errString = `${errString} ${apiResp.statusText}`
        }
    } else {
        errString = `Request failure: \n  -> ${errs.join(`\n  -> `)}`
    }
    logger.info(util.inspect(apiResp.body))
    throw new exceptionalError(errString)
}


export function overrideExpTime(newTime: number){
    if(isProduction){
        throw new exceptionalError("This aint for you production!")
    }
    expiry = newTime
}


export async function rawRequest(endpoint: string, method:string){
    if(isProduction){
        throw new exceptionalError("This aint for you production!")
    }
    return await performApiCall(endpoint, method, {})
}