import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import fetch from 'node-fetch';
import fetchMock from 'fetch-mock';
const expect = chai.expect;
chai.use(chaiAsPromised);

import * as smrt from '../smrtfill.js'
import * as authCases from './testDetails/smrt.cases.auth.js'
import * as rslConstants from '../smrtfill.constants.js';
import * as getTestCases from './testDetails/smrt.cases.get.js'


describe('I AM SO SMRT', async() => {
    before(async() => {
        fetchMock.restore();
        fetchMock.post(/auth/i, async (url, opts) => {
            return authCases.checkAuthReq(url, opts, 1)
        });
        fetchMock.get(/./, async (url, opts) => {
            return getTestCases.getReq(url, opts);
        });
        fetchMock.post(/orders/i, async (url, opts) => {
            return getTestCases.checkPostBody(url, opts);
        });
    });

    context('Gets orders by RSL id', async() => {
        it('Validates', async() => {
            await expect(smrt.getShipment({rslReference: '13123fasfasf'})).to.be.rejectedWith('RSL IDs are numeric')
            await expect(smrt.getShipment({rslReference: '13123', orderNumber: "asdsadassd"})).to.be.rejectedWith('rslReference XOR orderNumber')
            //@ts-ignore
            await expect(smrt.getShipment({rslReference: null})).to.be.rejectedWith('RSL ID must be non-empty')
        })
        
        it('Handles the no-match 400 case', async () => {
            const idErr = '400';
            await expect(smrt.getShipment({rslReference: idErr})).to.be.rejectedWith('No match for RSL ID 400')
        })

        it('Handles the no-match 404 case', async () => {
            const idErr = '404';
            await expect(smrt.getShipment({rslReference: idErr})).to.be.rejectedWith('No match for RSL ID 404')
        })

        it('Gets orders by RSL ID', async () => {
            const idAccepted = '67298';
            let acceptedOrder = await smrt.getShipment({rslReference: idAccepted}) as rslConstants.niceShipment;
            expect(acceptedOrder.orderNumber).to.equal("NEWBDORDER123");
            expect(acceptedOrder.rslReference).to.equal("123456789")
            expect(acceptedOrder.requestedMethod).to.equal("2nd Day Air")
            expect(acceptedOrder.status).to.equal("Accepted")
            expect(acceptedOrder.shipments).to.be.empty;
            expect(acceptedOrder.receiver).to.deep.equal({
                name: "Chris Nevin",
                phone: "",
                email: "cnevin@yert.ca",
                addressOne: "56 Brodie St",
                addressTwo: null,
                addressThree: null,
                company: null,
                city: "Mount Pearl",
                state: "CA",
                country: "US",
                zip: "54321"
            })
            expect(acceptedOrder.reqItems).to.deep.equal([
                {
                    sku: 'BD2',
                    qty: 5,
                    description: "BlueDriver2 Bluetooth Pro OBDII Scan Tool"
                },
                {
                    sku: '2746-B00652G4TS',
                    qty: 2,
                    description: "BlueDriver"
                }
            ])

            const idFulfilling = '8963217'
            let fulfilledOrder = await smrt.getShipment({rslReference: idFulfilling}) as rslConstants.niceShipment;
            expect(fulfilledOrder.status).to.equal("InFulfillment")
            expect(String(fulfilledOrder.dateCreated)).to.equal(String(new Date("2023-07-17T04:00:18Z")))
        })

        it('Catches shipment qty mismatches', async() => {
            // 6789321
            await expect(smrt.getShipment({rslReference: 6789321})).to.be.rejectedWith('Order 6789321 had 5 in the request, but 3 were shipped')
        })

        it('Catches shipment carrier detail mismatches', async() => {
            // 999888111
            await expect(smrt.getShipment({rslReference: 999888111})).to.be.rejectedWith('Order 999888111 has carrier USPS for service 2nd Day Air when it should be UPS')
        })

        it('Catches incomplete tracking info', async() => {
            // 9876541
            await expect(smrt.getShipment({rslReference: "9876541"})).to.be.rejectedWith('Order 9876541 package 536831692 had incomplete shipping deets')
        })
    })


    context('Gets orders by our order #', async() => {
        it('Validates', async() => {
            await expect(smrt.getShipment({orderNumber: ''})).to.be.rejectedWith('Order number must be non-empty')
            //@ts-ignore
            await expect(smrt.getShipment({orderNumber: null})).to.be.rejectedWith('Order number must be non-empty')
            await expect(smrt.getShipment({})).to.be.rejectedWith('Some filter criteria is required')
        })

        it('Handles the no-match case', async() => {
            const orderNumber = 'noOrderNumberMatch';
            await expect(smrt.getShipment({orderNumber: orderNumber})).to.be.rejectedWith('Got 0 results for noOrderNumberMatch')
        })

        it('Gets orders by our order #', async() => {
            const orderNumber = 'aValidOrderNumber';
            let order = await smrt.getShipment({orderNumber: orderNumber}) as rslConstants.niceShipment;
            expect(order.shipments).to.have.lengthOf(2)
            expect(order.orderNumber).to.equal(orderNumber);
            expect(order.rslReference).to.equal("51341115");
            expect(order.requestedMethod).to.equal("Priority Mail Small Flat Rate Box")
            expect(order.status).to.equal("Shipped")
            expect(order.shipments).to.deep.equal(
                [
                    {
                        shipmentId: "536831692",
                        dimensions: "10x6x1",
                        packageQty: 3,
                        weight: 0.45,
                        shipDate: new Date("2023-07-14T13:38:00Z"),
                        carrier: "USPS",
                        shipService: "Priority Mail",
                        niceService: "Express",
                        tracking: "9405511206209899177740",
                        trackURL: "https://tools.usps.com/go/TrackConfirmAction.action?tRef=fullpage&tLc=1&text28777=&tLabels=9405511206209899177740",
                        containedSkus: {
                            "2746-B00652G4TS": 3
                        }
                    },
                    {
                        shipmentId: "536831693",
                        dimensions: "10x6x1",
                        packageQty: 2,
                        weight: 0.30,
                        shipDate: new Date("2023-07-14T13:52:00Z"),
                        carrier: "USPS",
                        shipService: "Priority Mail",
                        niceService: "Express",
                        tracking: "9405511206209899177741",
                        trackURL: "https://tools.usps.com/go/TrackConfirmAction.action?tRef=fullpage&tLc=1&text28777=&tLabels=9405511206209899177741",
                        containedSkus: {
                            "2746-B00652G4TS": 2
                        }
                    }
                ]
            )

        })

        it('Catches >1 match', async() => {
            const orderNumber = 'tooManyResponses';
            await expect(smrt.getShipment({orderNumber: orderNumber})).to.be.rejectedWith('Got 2 results for tooManyResponses')
        })
    })


    context('Creates orders', async() => {
        it('Validates', async() => {
            //@ts-ignore
            await expect(smrt.createShipment(null, {}, {}, "")).to.be.rejectedWith("Order Number Is Required");
            //@ts-ignore
            await expect(smrt.createShipment("", {}, {}, "")).to.be.rejectedWith("Order Number Is Required");
            //@ts-ignore
            await expect(smrt.createShipment("someOrder", {}, {}, "")).to.be.rejectedWith("Missing mandatory customer address field name");
            //@ts-ignore
            await expect(smrt.createShipment("someOrder", {}, {name:"1", addressOne:"2", city:"3", state: "4", country: "5", zip: "6"}, "")).to.be.rejectedWith("At least one SKU:QTY entry is required");
            //@ts-ignore
            await expect(smrt.createShipment("someOrder", {"": null}, {name:"1", addressOne:"2", city:"3", state: "4", country: "5", zip: "6"}, "")).to.be.rejectedWith("Empty or null skus found in shipment item list");
            //@ts-ignore
            await expect(smrt.createShipment("someOrder", {"asdsad": {qty: "5"}}, {name:"1", addressOne:"2", city:"3", state: "4", country: "5", zip: "6"}, "")).to.be.rejectedWith("Non-numeric qty found for sku");
             //@ts-ignore
             await expect(smrt.createShipment("someOrder", {"asdsad": {qty: 5}}, {name:"1", addressOne:"2", city:"3", state: "4", country: "5", zip: "6"}, "")).to.be.rejectedWith("Shipping method is required");
             //@ts-ignore
             await expect(smrt.createShipment("someOrder", {"asdsad": {qty: 5}}, {name:"1", addressOne:"2", city:"3", state: "4", country: "5", zip: "6"}, "pigeon")).to.be.rejectedWith("Invalid shipping method selected");
        })


        it('Creates a shipment with nice shipping name', async () => {
            const aNiceNewOrder = {
                orderNumber: "shinyBrandNewOrder!",
                person:{
                    name: "Chris Nevin",
                    addressOne: "56 Brodie Street",
                    addressTwo: "Suite 102",
                    addressThree: "3rd Floor",
                    email: "cnevin@yert.ca",
                    phone: "709-749-1029",
                    company: "LEMOOR",
                    city: "Mount Pearl",
                    state: "CA",
                    country: "US",
                    zip: "12345"
                },
                order: {BD3: {qty: 4}, iceCream: {qty: 9, unitPrice: 9999}}
            }
            let newOrderId = await smrt.createShipment(aNiceNewOrder.orderNumber, aNiceNewOrder.order, aNiceNewOrder.person, "Express", "");
            expect(newOrderId).to.equal("1234567898765432")
        })


        it('Tries to create a shipment but cant validate it - throws exception', async function() {
            // this.timeout doesn't like arrow functions!
            
            this.timeout(6000);
            const orderNoMatch = {
                orderNumber: "someOrderFailsValidation",
                person: {name:"1", addressOne:"2", city:"3", state: "4", country: "5", zip: "6"},
                order: {"BD3": {qty: 25}}
            }
            await expect(smrt.createShipment(orderNoMatch.orderNumber, orderNoMatch.order, orderNoMatch.person, "Priority", "")).to.be.rejectedWith("Failed to validate new shipment")
        })
    })


})